#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

#define MINREAL -1024.0
#define MAXREAL 1024.0

#include <sys/time.h>

double gettime(void) {
   struct timeval tv;
   gettimeofday(&tv, NULL);
   return tv.tv_sec + 1e-6 * tv.tv_usec;
}

void matFillRand(long M, long N, double *mat) {
   long i, j;

   for(i = 0; i < M; i ++)
      for(j = 0; j < N; j ++)
         mat[i * N + j] = (rand() / (double) RAND_MAX)*(MAXREAL - MINREAL) + MINREAL;
}

void vecFillRand(long N, double *vec) {
   long i;

   for(i = 0; i < N; i++) 
      vec[i] = (rand() / (double) RAND_MAX)*(MAXREAL - MINREAL) + MINREAL;
}

void matVecMul(long M, long N, double *a, double *b, double *c) {
   long i, j, k;

   for(i = 0; i < N; i++) {
     c[i] = 0.0;
   }

   for(i = 0; i < M; i++)
      for(j = 0; j < N; j++)
         c[i] += a[i * N + j] * b[j]; 
}

int main(int argc, char **argv) {
   long M, N;
   double *A, *B, *C, t;

   srand(time(NULL));

   if (argc == 3) {
     M = atoi(argv[1]);
     N = atoi(argv[2]);
   } else {
     printf("M?"); scanf("%ld", &M);
     printf("N?"); scanf("%ld", &N);
   }
   
   A = (double *) malloc(M * N * sizeof(double));
   B = (double *) malloc(N * sizeof(double));
   C = (double *) malloc(M * sizeof(double));
   
   matFillRand(M, N, A);
   vecFillRand(N, B);
  
   t = gettime();
   matVecMul(M, N, A, B, C);
   t = gettime() - t;

   fprintf(stdout, "%ld\t%ld\t%lf\n", M, N, t);
   fflush(stdout);

   free(A);
   free(B);
   free(C);

   return EXIT_SUCCESS;
}
